###############################################################################
# R code - missing data infill algorithm #
# This code is suitable for any time series data with weekly periodic activitie
# This code also accounts for three repeating entries (i.e. with same time stamp)
# This code requires users to organise input files in prescribed format(Section 2)
# The code will generate two types of output files: 
# Output 1 - removing repeated entries and infilling missng values with NA
# Output 2 - Missing  entries and infilled using our iterative-logical algorithm
###############################################################################

rm(list=ls(all=TRUE))    # removing all the objects present in the workspace
options(max.print = 99999999)              # setting limit for max print
#options(max.print = .Machine$integer.max) # setting limit for large dataset 

###############################################################################
# 1. Setting path for working, reading input, and writing output files 
###############################################################################

WD<-"C:/Project/Data"                     # setting path for working directory
ID_01<-"C:/Project/Data/Input"            # setting path for Input directory
OD_01<-"C:/Project/Data/Output"           # setting path for Output directory

setwd(WD)                                 # setting R in the Working directory

############################################################################
# 2. User-specified parameter setting. Please change as required           #
# No need to make any changes after this step
############################################################################

NameInputFile1<-"RawData.csv" 
# Name of input raw data file (*.csv format). 
# The RawData.csv file should have data organised in two columns 
# Column1: Data and Time stamp for the observed raw record.
# Column2 : Observed variables with missing entires (e.g. electricity demand)

NameInputFile2<-"Time_Data.csv" 
# Name of user specified format of Date/Time file (*.csv format)
# This file should have equidistance Data and Time stamp with no missing 
# points and in the same style/format as Column 1 of "RawData.csv". 

NameOutputFile1<-"Infill_MissingData_Nan.csv" 
# User-specified name of output file with missing points infilled with NA

NameOutputFile2<-"Infill_MissingData_LogicalM.csv" 
# Name of output file with all missing points infilled using algorithm

############################################################################
# Users are not required to make any changes after this step, however they #
# can follow the printed output and notes in the code to check the perform-#
# of the code step by step                                                 #
############################################################################

############################################################################
# 3. Reading input files                                                   #
############################################################################

# 3.1. Reading raw electrcitiy demand data with missing values

setwd(ID_01)
InFile1A<-read.csv(NameInputFile1, header = TRUE)
attach(InFile1A)
dim(InFile1A);  
ls(InFile1A)

# 3.2. Reading input files (Date/Time data) 
setwd(ID_01)
InFile1B<-read.csv(NameInputFile2, header = TRUE)
attach(InFile1B)
# dim(InFile1B); 
ls(InFile1B)

Len1A<-length(InFile1A[,1])
Len1B<-length(InFile1B[,1])

############################################################################
# 4. Infilling missing value with NaN and removing repeating data          #
############################################################################

DateeC<-as.character(InFile1B[,1]) 
DateC<-as.character(InFile1A[,1])

# Check - make sure last data point in Datec and DateeC match #
#length(DateeC); length(DateC) ; DateeC[1:5]; DateC[1:5] 

RReading<- InFile1A[,2] ; length(RReading) ; RReading[1:5]

LenDatee<-length(DateeC)   
# for one minutely data over 61 days - 60Min*24hours*61days = 60*24*61=87840

LenDate<-length(DateC)    # actual length of dataset  
DateC[LenDate]

# 4.1 Percentage of missing points 
 TotalMissingData<-LenDatee-LenDate; TotalMissingData
 PerMissingData<-(TotalMissingData/LenDatee)*100; PerMissingData
 print(c("Total no of Missing Data", TotalMissingData))
 print(c("Percentage of Missing Data", PerMissingData))

# 4.2 Infilling missing points with Nan 

if(LenDate<LenDatee){
  
  countii<--1;   countij<-0; countjj<--1; countjk<-0; icou1<-0; icou2<-0; icou3<-0
  countijk<-0; countijk1<-0
  
  for(ii in 2:LenDatee){
    #  for(ii in 2:3000){
    
    Datee0<-DateeC[ii]
    Date0<-DateC[ii]
    Reading0<-RReading[ii]
    
    Date_1<-DateC[ii-1]
    Date_p1<-DateC[ii+1]
    
    Read1<-RReading[ii]+RReading[ii+1]
    Read2<-RReading[ii]+RReading[ii+1]+RReading[ii+2]
    jj<-ii-1
    jj1<-ii+1
    jj2<-ii+2
    jj3<-ii+3
    
    #case 1 - repeating value  up to 3    
    if(Date0==Date_1&Date0==Date_p1){
      
      countijk<-countijk+2
      countijk1<-LenDate-countijk+icou2
      
      Date1<-NULL;Date2<-NULL
      Date1<-DateC[1:jj]
      Date2<-DateC[jj2:countijk1]
      DateC<-NULL
      DateC<-c(Date1,Date2)
      
      Reading1<-NULL; Reading2<-NULL
      Reading1<-RReading[1:jj]
      Reading2<-c(Read2, RReading[jj3:countijk1])
      RReading<-NULL
      RReading<-c(Reading1,Reading2)
      
      icou3<-icou3+2
      print(c("repeat3",ii, countijk, countijk1, icou3))
    }
    
    #case 2 - repeating value  up to 2 
    
    else if(Date0==Date_1){
      
      countii<-countii+1
      countij<-LenDate-countii+icou2-icou3
      
      Date1<-NULL;Date2<-NULL
      Date1<-DateC[1:jj]
      Date2<-DateC[jj1:countij]
      DateC<-NULL
      DateC<-c(Date1,Date2)
      
      Reading1<-NULL; Reading2<-NULL
      Reading1<-RReading[1:jj]
      Reading2<-c(Read1, RReading[jj2:countij])
      RReading<-NULL
      RReading<-c(Reading1,Reading2)
      
      icou1<-icou1+1
      print(c("repeat",ii, countii, countij, icou1))
    }
    
    # Case 3 - replacing missig value with -9.99 (Nan)  
    else if(Datee0!=Date0){
      countjj<-countjj+1
      countjk<-LenDate-icou1+countjj-icou3
      
      Date1<-NULL;Date2<-NULL
      Date1<-c(DateC[1:jj],Datee0)
      Date2<-DateC[ii:countjk]
      DateC<-NULL
      DateC<-c(Date1,Date2)
      
      
      Reading1<-NULL; Reading2<-NULL
      Reading1<-c(RReading[1:jj],-9.99)
      Reading2<-RReading[ii:countjk]
      RReading<-NULL
      RReading<-c(Reading1,Reading2)
      
      icou2<-icou2+1
      print(c("missing",ii,countjj,countjk))
    }
    
    print(c(ii,length(DateC),length(RReading)))
  }
  
}  

Date0; Date_1; Date_p1
length(DateeC); length(DateC); length(RReading) 
DateC[87840]; DateeC[87840]; RReading[87840]

Time_SP1<-c(DateC)
RReading_SP1<-c(RReading)

# Replacing any RReading coded as NA 

RReading_SP1[is.na(RReading_SP1)]<--9.99
RReading_SP1[is.nan(RReading_SP1)]<--9.99

### 4.3 Writing updated file with equal timestamp ###
###  missing values coded as -9.99  ### 
OutFileA<-NULL
OutFileA<-cbind(Time_SP1, RReading_SP1)
Header<-cbind("Time", "Meter_Reading")

Output1<-rbind(Header,OutFileA)
Output1<-t(Output1)
dim(Output1)


setwd(OD_01)  
write(Output1,file=NameOutputFile1,ncolumns=2,sep=",",append = F)

###############################################################
# 5. Infilling NaN/-9.99 with previous values a week ago or after
###############################################################

### 5.1 Counting total Nan values in data
j<-0
for(i in 1:LenDatee){
  Val0<-RReading_SP1[i]
  if(Val0<0.0){
    j=j+1
  }
}
RReading[58402]
j # number of Nan that has been recoded as -9.99

### 5.2 Infilling Nan/-9.99 values using a designed 10 run strategy
### RUN 1 to replace -9.99 with a previous/later week value 
if(j>0){
jj1<-0; jj2<-0; jj3<-0; RReading_SP2<-NULL

RReading_SP2[1]<-RReading_SP1[1]
# no of points in a week = 60*24*7 = 10080
for(i in 2:10080){
  
  Val1<-RReading_SP1[i-1]
  Val2<-RReading_SP1[i]
  
  if(Val2<0.0){
    jj1<-jj1+1
    RReading_SP2[i]<- RReading_SP1[i+10080]
    print(c("jj1", jj1, RReading_SP2[i]))
  }
  else if (Val2>0.0&Val1<0.0){
    jj2<-jj2+1
    RReading_SP2[i]<-RReading_SP1[i+10080]
    print(c("jj2", jj2, RReading_SP2[i]))
  }
  else if (Val2>0.0){
    jj3<-jj3+1
    RReading_SP2[i]<-RReading_SP1[i]
    print(c("jj3", jj3, RReading_SP2[i]))
  }
}
jj1; jj2; jj3

jj1+jj2+jj3

for(i in 10081:LenDatee){
  Val1<-RReading_SP1[i-1]
  Val2<-RReading_SP1[i]
  
  if(Val2<0.0){
    jj1<-jj1+1
    RReading_SP2[i]<- RReading_SP1[i-10080]
    print(c("jj1", jj1, RReading_SP2[i]))
  }
  else if (Val2>0.0&Val1<0.0){
    jj2<-jj2+1
    RReading_SP2[i]<-RReading_SP1[i-10080]
    print(c("jj2", jj2, RReading_SP2[i]))
  }
  else if (Val2>0.0){
    jj3<-jj3+1
    RReading_SP2[i]<-RReading_SP1[i]
    print(c("jj3", jj3, RReading_SP2[i]))
  }
  
}

jj1; jj2; jj3

jj1+jj2+jj3 # no of recoded Nan replaced with previous values

# j should be equal to jj1+jj2+jj3

RReading_SP2[is.na(RReading_SP2)]<--9.99

jj4<-0
for(i in 1:LenDatee){
  Val<-RReading_SP2[i]
  if(Val<0.0){
    jj4=jj4+1
    print(c(jj4, "Nan values not replaced run code below"))
}
#  print(i)
}
jj4 # No of remaining Nan
# This happen as if replacement value itself is a Nan (uncomment this )
}

### Run 2 to replace remaining Nan ###########################################
if(jj4>0){
  
kk1<-0; kk2<-0; kk3<-0; RReading_SP3<-NULL

RReading_SP3[1]<-RReading_SP2[1]
for(i in 2:10080){
  
  Val1<-RReading_SP2[i-1]
  Val2<-RReading_SP2[i]
  
  if(Val2<0.0){
    kk1<-kk1+1
    RReading_SP3[i]<- RReading_SP2[i+10080]
    print(c("kk1", kk1, RReading_SP3[i]))
  }
  else if (Val2>0.0&Val1<0.0){
    kk2<-kk2+1
    RReading_SP3[i]<-RReading_SP2[i+10080]
    print(c("kk2", kk2, RReading_SP3[i]))
  }
  else if (Val2>0.0){
    kk3<-kk3+1
    RReading_SP3[i]<-RReading_SP2[i]
    print(c("kk3", kk3, RReading_SP3[i]))
  }
}
kk1; kk2; kk3

kk1+kk2+kk3

for(i in 10081:LenDatee){
  Val1<-RReading_SP2[i-1]
  Val2<-RReading_SP2[i]
  
  if(Val2<0.0){
    kk1<-kk1+1
    RReading_SP3[i]<- RReading_SP2[i-10080]
    print(c("kk1", kk1, RReading_SP3[i]))
  }
  else if (Val2>0.0&Val1<0.0){
    kk2<-kk2+1
    RReading_SP3[i]<-RReading_SP2[i-10080]
    print(c("kk2", kk2, RReading_SP3[i]))
  }
  else if (Val2>0.0){
    kk3<-kk3+1
    RReading_SP3[i]<-RReading_SP2[i]
    print(c("kk3", kk3, RReading_SP3[i]))
  }
  
}

kk1; kk2; kk3

kk1+kk2+kk3 # no of recoded Nan replaced with previous values

RReading_SP3[is.na(RReading_SP3)]<--9.99

kk4<-0
for(i in 1:LenDatee){
  Val<-RReading_SP3[i]
  if(Val<0.0){
    kk4=kk4+1
    print(c(kk4, "Nan values not replaced run code below"))
  }
  print(i)
}
kk4 # no of remaining Nan 

}

#### Run 3 ###########################################
if(kk4>0){
  
  MM1<-0; MM2<-0; MM3<-0; RReading_SP4<-NULL
  
  RReading_SP4[1]<-RReading_SP3[1]
  for(i in 2:10080){
    
    Val1<-RReading_SP3[i-1]
    Val2<-RReading_SP3[i]
    
    if(Val2<0.0){
      MM1<-MM1+1
      RReading_SP4[i]<- RReading_SP3[i+10080]
      print(c("MM1", MM1, RReading_SP4[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      MM2<-MM2+1
      RReading_SP4[i]<-RReading_SP3[i+10080]
      print(c("MM2", MM2, RReading_SP4[i]))
    }
    else if (Val2>0.0){
      MM3<-MM3+1
      RReading_SP4[i]<-RReading_SP3[i]
      print(c("MM3", MM3, RReading_SP4[i]))
    }
  }
  MM1; MM2; MM3
  
  MM1+MM2+MM3
  
  for(i in 10081:LenDatee){
    Val1<-RReading_SP3[i-1]
    Val2<-RReading_SP3[i]
    
    if(Val2<0.0){
      MM1<-MM1+1
      RReading_SP4[i]<- RReading_SP3[i-10080]
      print(c("MM1", MM1, RReading_SP4[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      MM2<-MM2+1
      RReading_SP4[i]<-RReading_SP3[i-10080]
      print(c("MM2", MM2, RReading_SP4[i]))
    }
    else if (Val2>0.0){
      MM3<-MM3+1
      RReading_SP4[i]<-RReading_SP3[i]
      print(c("MM3", MM3, RReading_SP4[i]))
    }
    
  }
  
  MM1; MM2; MM3
  
  MM1+MM2+MM3 # no of recoded Nan replaced with previous values

  RReading_SP4[is.na(RReading_SP4)]<--9.99
  MM4<-0
  for(i in 1:LenDatee){
    Val<-RReading_SP4[i]
    if(Val<0.0){
      MM4=MM4+1
      print(c(MM4, "Nan values not replaced run code below"))
    }
  }
  
}

### Run 4 ###########################################
### IN THIS CASE REPLACING WITH TWO LATER WEEK VALUES 
if(MM4>0){
  
  NN1<-0; NN2<-0; NN3<-0; RReading_SP5<-NULL
  
  RReading_SP5[1]<-RReading_SP4[1]
  for(i in 2:20160){
    
    Val1<-RReading_SP4[i-1]
    Val2<-RReading_SP4[i]
    
    if(Val2<0.0){
      NN1<-NN1+1
      RReading_SP5[i]<- RReading_SP4[i+20160]
      print(c("NN1", NN1, RReading_SP5[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      NN2<-NN2+1
      RReading_SP5[i]<-RReading_SP4[i+20160]
      print(c("NN2", NN2, RReading_SP5[i]))
    }
    else if (Val2>0.0){
      NN3<-NN3+1
      RReading_SP5[i]<-RReading_SP4[i]
      print(c("NN3", NN3, RReading_SP5[i]))
    }
  }
  NN1; NN2; NN3
  
  NN1+NN2+NN3
  
  for(i in 20161:LenDatee){
    Val1<-RReading_SP4[i-1]
    Val2<-RReading_SP4[i]
    
    if(Val2<0.0){
      NN1<-NN1+1
      RReading_SP5[i]<- RReading_SP4[i-20160]
      print(c("NN1", NN1, RReading_SP5[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      NN2<-NN2+1
      RReading_SP5[i]<-RReading_SP4[i-20160]
      print(c("NN2", NN2, RReading_SP5[i]))
    }
    else if (Val2>0.0){
      NN3<-NN3+1
      RReading_SP5[i]<-RReading_SP4[i]
      print(c("NN3", NN3, RReading_SP5[i]))
    }
    
  }
  
  NN1; NN2; NN3
  
  NN1+NN2+NN3 # no of recoded Nan replaced with previous values

  RReading_SP5[is.na(RReading_SP5)]<--9.99
  NN4<-0
  for(i in 1:LenDatee){
    Val<-RReading_SP5[i]
    if(Val<0.0){
      NN4=NN4+1
      print(c(NN4, "Nan values not replaced run code below"))
    }
  }
  
}

### Run 5 ###########################################
### IN THIS CASE REPLACING AGAIN WITH TWO LATER WEEK VALUES 
if(NN4>0){
  
  PP1<-0; PP2<-0; PP3<-0; RReading_SP6<-NULL
  
  RReading_SP6[1]<-RReading_SP5[1]
  for(i in 2:20160){
    
    Val1<-RReading_SP5[i-1]
    Val2<-RReading_SP5[i]
    
    if(Val2<0.0){
      PP1<-PP1+1
      RReading_SP6[i]<- RReading_SP5[i+20160]
      print(c("PP1", PP1, RReading_SP6[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      PP2<-PP2+1
      RReading_SP6[i]<-RReading_SP5[i+20160]
      print(c("PP2", PP2, RReading_SP6[i]))
    }
    else if (Val2>0.0){
      PP3<-PP3+1
      RReading_SP6[i]<-RReading_SP5[i]
      print(c("PP3", PP3, RReading_SP6[i]))
    }
  }
  PP1; PP2; PP3
  
  PP1+PP2+PP3
  
  for(i in 20161:LenDatee){
    Val1<-RReading_SP5[i-1]
    Val2<-RReading_SP5[i]
    
    if(Val2<0.0){
      PP1<-PP1+1
      RReading_SP6[i]<- RReading_SP5[i-20160]
      print(c("PP1", PP1, RReading_SP6[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      PP2<-PP2+1
      RReading_SP6[i]<-RReading_SP5[i-20160]
      print(c("PP2", PP2, RReading_SP6[i]))
    }
    else if (Val2>0.0){
      PP3<-PP3+1
      RReading_SP6[i]<-RReading_SP5[i]
      print(c("PP3", PP3, RReading_SP6[i]))
    }
    
  }
  
  PP1; PP2; PP3
  
  PP1+PP2+PP3 # no of recoded Nan replaced with previous values
  RReading_SP6[is.na(RReading_SP6)]<--9.99
  PP4<-0
  for(i in 1:LenDatee){
    Val<-RReading_SP6[i]
    if(Val<0.0){
      PP4=PP4+1
      print(c(PP4, "Nan values not replaced run code below"))
    }
  }
  PP4 
}

### Run 6 ###########################################
### IN THIS CASE REPLACING WITH THREE LATER WEEK VALUES
### USE ONLY IF YOU HAVE DATA MORE THAN 6 WEEKS
if(PP4>0){
  
  QQ1<-0; QQ2<-0; QQ3<-0; RReading_SP7<-NULL
  
  RReading_SP7[1]<-RReading_SP6[1]
  for(i in 2:30240){
    
    Val1<-RReading_SP6[i-1]
    Val2<-RReading_SP6[i]
    
    if(Val2<0.0){
      QQ1<-QQ1+1
      RReading_SP7[i]<- RReading_SP6[i+30240]
      print(c("QQ1", QQ1, RReading_SP7[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      QQ2<-QQ2+1
      RReading_SP7[i]<-RReading_SP6[i+30240]
      print(c("QQ2", QQ2, RReading_SP7[i]))
    }
    else if (Val2>0.0){
      QQ3<-QQ3+1
      RReading_SP7[i]<-RReading_SP6[i]
      print(c("QQ3", QQ3, RReading_SP7[i]))
    }
  }
  QQ1; QQ2; QQ3
  
  QQ1+QQ2+QQ3
  
  for(i in 30241:LenDatee){
    Val1<-RReading_SP6[i-1]
    Val2<-RReading_SP6[i]
    
    if(Val2<0.0){
      QQ1<-QQ1+1
      RReading_SP7[i]<- RReading_SP6[i-30240]
      print(c("QQ1", QQ1, RReading_SP7[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      QQ2<-QQ2+1
      RReading_SP7[i]<-RReading_SP6[i-30240]
      print(c("QQ2", QQ2, RReading_SP7[i]))
    }
    else if (Val2>0.0){
      QQ3<-QQ3+1
      RReading_SP7[i]<-RReading_SP6[i]
      print(c("QQ3", QQ3, RReading_SP7[i]))
    }
    
  }
  
  QQ1; QQ2; QQ3
  
  QQ1+QQ2+QQ3 # no of recoded Nan replaced with previous values
  RReading_SP7[is.na(RReading_SP7)]<--9.99 
  QQ4<-0
  for(i in 1:LenDatee){
    Val<-RReading_SP7[i]
    if(Val<0.0){
      QQ4=QQ4+1
      print(c(QQ4, "Nan values not replaced run code below"))
    }
  }
  QQ4 
}

### Run 7 ###########################################
### IN THIS CASE AGAIN REPLACING WITH TWO LATER WEEK VALUES
### USE ONLY IF YOU HAVE DATA MORE THAN 6 WEEKS
if(QQ4>0){
  
  RR1<-0; RR2<-0; RR3<-0; RReading_SP8<-NULL
  
  RReading_SP8[1]<-RReading_SP7[1]
  for(i in 2:20160){
    
    Val1<-RReading_SP7[i-1]
    Val2<-RReading_SP7[i]
    
    if(Val2<0.0){
      RR1<-RR1+1
      RReading_SP8[i]<- RReading_SP7[i+20160]
      print(c("RR1", RR1, RReading_SP8[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      RR2<-RR2+1
      RReading_SP8[i]<-RReading_SP7[i+20160]
      print(c("RR2", RR2, RReading_SP8[i]))
    }
    else if (Val2>0.0){
      RR3<-RR3+1
      RReading_SP8[i]<-RReading_SP7[i]
      print(c("RR3", RR3, RReading_SP8[i]))
    }
  }
  RR1; RR2; RR3
  
  RR1+RR2+RR3
  
  for(i in 20161:LenDatee){
    Val1<-RReading_SP7[i-1]
    Val2<-RReading_SP7[i]
    
    if(Val2<0.0){
      RR1<-RR1+1
      RReading_SP8[i]<- RReading_SP7[i-20160]
      print(c("RR1", RR1, RReading_SP8[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      RR2<-RR2+1
      RReading_SP8[i]<-RReading_SP7[i-20160]
      print(c("RR2", RR2, RReading_SP8[i]))
    }
    else if (Val2>0.0){
      RR3<-RR3+1
      RReading_SP8[i]<-RReading_SP7[i]
      print(c("RR3", RR3, RReading_SP8[i]))
    }
    
  }
  
  RR1; RR2; RR3
  
  RR1+RR2+RR3 # no of recoded Nan replaced with previous values
  RReading_SP8[is.na(RReading_SP8)]<--9.99
  RR4<-0
  for(i in 1:LenDatee){
    Val<-RReading_SP8[i]
    if(Val<0.0){
      RR4=RR4+1
      print(c(RR4, "Nan values not replaced run code below"))
    }
  }
  RR4 
}

### Run 9 ###########################################
### IN THIS CASE AGAIN REPLACING WITH ONE LATER WEEK VALUES
if(RR4>0){
  
  SS1<-0; SS2<-0; SS3<-0; RReading_SP9<-NULL
  
  RReading_SP9[1]<-RReading_SP8[1]
  for(i in 2:10800){
    
    Val1<-RReading_SP8[i-1]
    Val2<-RReading_SP8[i]
    
    if(Val2<0.0){
      SS1<-SS1+1
      RReading_SP9[i]<- RReading_SP8[i+10800]
      print(c("SS1", SS1, RReading_SP9[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      SS2<-SS2+1
      RReading_SP9[i]<-RReading_SP8[i+10800]
      print(c("SS2", SS2, RReading_SP9[i]))
    }
    else if (Val2>0.0){
     SS3<-SS3+1
      RReading_SP9[i]<-RReading_SP8[i]
      print(c("SS3", SS3, RReading_SP9[i]))
    }
  }
  SS1; SS2; SS3
  
  SS1+SS2+SS3
  
  for(i in 10801:LenDatee){
    Val1<-RReading_SP8[i-1]
    Val2<-RReading_SP8[i]
    
    if(Val2<0.0){
      SS1<-SS1+1
      RReading_SP9[i]<- RReading_SP8[i-10800]
      print(c("SS1", SS1, RReading_SP9[i]))
    }
    else if (Val2>0.0&Val1<0.0){
      SS2<-SS2+1
      RReading_SP9[i]<-RReading_SP8[i-10800]
      print(c("SS2", SS2, RReading_SP9[i]))
    }
    else if (Val2>0.0){
      SS3<-SS3+1
      RReading_SP9[i]<-RReading_SP8[i]
      print(c("SS3", SS3, RReading_SP9[i]))
    }
    
  }
  
  SS1; SS2; SS3
  
  SS1+SS2+SS3 # no of recoded Nan replaced with previous values
  RReading_SP9[is.na(RReading_SP9)]<--9.99
  SS4<-0
  for(i in 1:LenDatee){
    Val<-RReading_SP9[i]
    if(Val<0.0){
      SS4=SS4+1
      print(c(SS4, "Nan values not replaced run code below"))
    }
  }
  SS4 
}

if(SS4==0){
  RReading_SP10<- RReading_SP9
  print("all nan replaced using weekly infill criterion")
} 

if (SS4!=0){
  print(c("Total nan in all data", SS4))
}
### 5.3 Writing updated file with continutes data and no missing value  ### 

  OutFileB<-NULL
  OutFileB<-cbind(Time_SP1, RReading_SP10)
  HeaderB<-cbind("Time", "Meter_Reading")
  
  Output2<-rbind(HeaderB,OutFileB)
  Output2<-t(Output2)
  dim(Output2)
  setwd(OD_01)  
  
  write(Output2,file=NameOutputFile2,ncolumns=2,sep=",",append = F)




